unit GeneralDBReportUnit;

// The program contains a procedure to throw any data set (i.e. a table or a
// query) on to a report.

// The clever thing about this routine, is that it will dynamically set the
// tab spacing to match the widths of the data within the query.

// You can use "as is" in any program. Just create a TQuery or TTable object,
// open it and send it to "DataSetReport" You have an instant report.

// While its very basic - it will in fact will deal with hundreds of different
// reports.

// As well as being a gernaral utility, this program illustrates the use of
// 'getWidth" and "setTabSpacing" to control tabs with the software rather than
// the wysiwyg tabs.


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, yapi, ExtCtrls, Db, DBTables,printers;

type
  TForm1 = class(TForm)
    Query1: TQuery;
    Paper: TyapiPaper;
    PrintButton: TButton;
    Body: TyapiText;
    PrintDialog1: TPrintDialog;
    Title: TyapiText;
    Header: TyapiText;
    PortraitButton: TButton;
    landscapeButton: TButton;
    FontButton: TButton;
    FontDialog: TFontDialog;
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Edit1: TEdit;
    procedure PrintButtonClick(Sender: TObject);
    procedure PortraitButtonClick(Sender: TObject);
    procedure landscapeButtonClick(Sender: TObject);
    procedure FontButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure DataSetReport(dataset:Tdataset; // The TTable or TQuery to be used for the report
                        Paper:Tyapipaper; // The paper component for the report
                        Body,             // The Font for the body of the report
                        fieldnames:TyapiText); // The font for the column Titles
var i,j,rowcount,colcount:Integer; // gerneral variables and dimensions of data set
  sdata:array of array of string;  // somewhere to hold the data prior to printing
  widths:array of double;          // widths of fields -all except the last one
  w:double;
  ColHeaders:tstringlist;          // a list of filed names for column headers
begin
   // before any field widths can be established, all of the result set must
   // be read into RAM to establish the maximum width of each column.
   colcount:=dataset.fieldcount;         // how many columns in the result set
   setlength(widths,colcount);           // variables to hold column widths
   ColHeaders:=tstringlist.create;       // Make a list to fetch the field names
   dataset.GetFieldNames(ColHeaders);    // get the field names
   rowcount:=0;
   while not dataset.eof do begin        // read each row - put into a 2 D array of strings
      setlength(sdata,colcount,rowcount+1);
      for i:=0 to colcount-1 do
          sdata[i,rowcount]:=dataset.fields[i].asstring;
      inc(rowcount);
      dataset.next;
   end;
   // All of the data is now in memory - now calculate the max widths of each field
   // field widths are multiplied by 1.1 to give a margin, and allow for font variation.
   for i:=0 to colcount-1 do begin
       widths[i]:=fieldnames.getWidth(ColHeaders[i])*1.1; // get widths of column headers
       for j:=0 to rowcount-1 do begin   // test each cell to see if it is wider ..
         w:=body.getWidth(sdata[i,j])*1.1;
         if w > widths[i] then widths[i]:=w; // if it is, update the width
       end;
   end;
   w:=0;
   for i:=0 to colcount-1 do     // get total width into "w"
       w:=w + widths[i];
   // test to see if report fits in margins
   if w>(paper.paperwidth - paper.MarginLeft - paper.MarginRight) then begin
       showmessage('Report is too wide for the paper');
   end;
   i:=length(widths);
   setlength(widths,i-1); // dispose of the last width - we dont need it.
   paper.setTabSpacing(widths);
   // Finished the tab sizing - Now do the report.
   paper.everypage;              // field names go on every page
   for j:=0 to colcount-1 do
      fieldnames.writeattab(ColHeaders[j],j);
   body.writeln;
   paper.drawline(body.font);    // underline on every page
   paper.thispage;               // back to the current page only
   for i:=0 to rowcount-1 do begin
     for j:=0 to colcount-1 do
        body.writeattab(sdata[j,i],j); // write each column at its tab
     body.writeln;
   end;
end;


procedure TForm1.PrintButtonClick(Sender: TObject);
begin
  // Empty the report first
  paper.clear;
  // put a title on the first page
  Title.writeln(Edit1.text);
  Title.writeln;
  // Put a small title on the other pages
  paper.otherpages;
  Body.writeln(Edit1.text+' (cont)');
  Body.writeln;
  // get the SQL statement
  Query1.SQL.text:=memo1.lines.text;
  Query1.open;
  // call the general purpose data set report function
  DatasetReport(Query1,paper,Body,Header);
  // All done - close the query
  Query1.close;
  paper.preview; // can use paper.print here if you don't want a preview
end;

procedure TForm1.PortraitButtonClick(Sender: TObject);
begin
 paper.paperOrientation:=yapPortrait;
end;

procedure TForm1.landscapeButtonClick(Sender: TObject);
begin
 paper.paperOrientation:=yaplandscape;
end;

procedure TForm1.FontButtonClick(Sender: TObject);
begin
   if FontDialog.execute then begin
       Body.font.assign(FontDialog.Font);
       Body.font.style:=Body.font.style-[fsUnderline];
       Header.font.assign(FontDialog.Font);
       Header.font.style:=Body.font.style+[fsBold];
       Title.font.name:=FontDialog.Font.name;
   end;
end;

end.
